/**
 * Ext GWT 3.0.0-rc2 - Ext for GWT
 * Copyright(c) 2007-2012, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.desktopapp.client.spreadsheet;

import java.util.LinkedList;
import java.util.List;

import com.google.gwt.cell.client.AbstractCell;
import com.google.gwt.cell.client.Cell;
import com.google.gwt.core.client.Scheduler;
import com.google.gwt.core.client.Scheduler.RepeatingCommand;
import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.event.logical.shared.ValueChangeHandler;
import com.google.gwt.safehtml.shared.SafeHtml;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.IdentityValueProvider;
import com.sencha.gxt.core.client.Style.SelectionMode;
import com.sencha.gxt.desktopapp.client.filemanager.images.Images;
import com.sencha.gxt.dnd.core.client.DND.Feedback;
import com.sencha.gxt.dnd.core.client.DndDropEvent;
import com.sencha.gxt.dnd.core.client.GridDragSource;
import com.sencha.gxt.dnd.core.client.GridDropTarget;
import com.sencha.gxt.widget.core.client.Window;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer.VerticalLayoutData;
import com.sencha.gxt.widget.core.client.event.ColumnMoveEvent;
import com.sencha.gxt.widget.core.client.event.ColumnMoveEvent.ColumnMoveHandler;
import com.sencha.gxt.widget.core.client.event.CompleteEditEvent;
import com.sencha.gxt.widget.core.client.event.CompleteEditEvent.CompleteEditHandler;
import com.sencha.gxt.widget.core.client.event.HideEvent;
import com.sencha.gxt.widget.core.client.event.HideEvent.HideHandler;
import com.sencha.gxt.widget.core.client.event.ShowEvent;
import com.sencha.gxt.widget.core.client.event.ShowEvent.ShowHandler;
import com.sencha.gxt.widget.core.client.form.TextField;
import com.sencha.gxt.widget.core.client.grid.CellSelectionModel;
import com.sencha.gxt.widget.core.client.grid.ColumnConfig;
import com.sencha.gxt.widget.core.client.grid.ColumnModel;
import com.sencha.gxt.widget.core.client.grid.Grid;
import com.sencha.gxt.widget.core.client.grid.RowNumberer;
import com.sencha.gxt.widget.core.client.grid.editing.ClicksToEdit;
import com.sencha.gxt.widget.core.client.grid.editing.GridInlineEditing;
import com.sencha.gxt.widget.core.client.selection.CellSelection;
import com.sencha.gxt.widget.core.client.selection.CellSelectionChangedEvent;
import com.sencha.gxt.widget.core.client.selection.CellSelectionChangedEvent.CellSelectionChangedHandler;

public class SpreadsheetViewImpl implements SpreadsheetView, HideHandler, ShowHandler, ColumnProvider {

  public static final String CELL_TOOL_TIP = "Enter a value or expression, for example:<br/>=((C2-B2)/B2)*100 or =SUM(B2:B10)<br/>Use $ for same row or column (e.g. B$ or $2).";

  private static final int DEFAULT_COLUMN_WIDTH = 100;

  private SpreadsheetPresenter spreadsheetPresenter;
  private Window window;
  private Grid<Row> grid;
  private Worksheet worksheet;
  private ColumnModel<Row> columnModel;
  private List<ColumnConfig<Row, ?>> columnConfigs;
  private RowNumberer<Row> rowNumberer;
  private IdentityValueProvider<Row> identityValueProvider;
  private GridInlineEditing<Row> gridInlineEditing;
  private VerticalLayoutContainer verticalLayoutContainer;
  private SpreadsheetToolBar spreadsheetToolBar;
  private Cell<String> displayCell;
  private CellSelectionModel<Row> cellSelectionModel;
  private GridDragSource<Row> gridDragSource;
  private GridDropTarget<Row> gridDropTarget;

  // TODO: Move non-view function from view to presenter

  public SpreadsheetViewImpl(SpreadsheetPresenter spreadsheetPresenter) {
    this.spreadsheetPresenter = spreadsheetPresenter;
  }

  @Override
  public Widget asWidget() {
    return getWindow();
  }

  @Override
  public void close() {
    getWindow().hide();
  }

  @Override
  public ColumnConfig<Row, Object> getColumn(int columnIndex) {
    return getGrid().getColumnModel().getColumn(columnIndex);
  }

  @Override
  public String getValue() {
    return getWorksheet().getValue();
  }

  @Override
  public Worksheet getWorksheet() {
    if (worksheet == null) {
      worksheet = new Worksheet(this);
    }
    return worksheet;
  }

  @Override
  public void onColumnCountChange(int newColumnCount) {
    getWorksheet().changeColumnCount(newColumnCount);
    displayGrid();
    getSpreadsheetPresenter().onSave();
  }

  @Override
  public void onHide(HideEvent event) {
    getSpreadsheetPresenter().unbind();
  }

  @Override
  public void onRowCountChange(int newRowCount) {
    getWorksheet().changeRowCount(newRowCount);
    getSpreadsheetPresenter().onSave();
  }

  @Override
  public void onShow(ShowEvent event) {
    getSpreadsheetPresenter().bind();
    getGrid().focus();
  }

  @Override
  public void onUpdateSelectedValues(String value) {
    CellSelection<Row> cell = getCellSelectionModel().getSelectCell();
    if (cell != null) {
      getWorksheet().setValueAbsolute(cell.getRow(), cell.getCell(), value);
      getGrid().getView().refresh(false);
      getSpreadsheetPresenter().onSave();
    }
  }

  @Override
  public void setColumnHeader(int columnIndex, SafeHtml header) {
    getGrid().getColumnModel().setColumnHeader(columnIndex, header);
  }

  @Override
  public void setTitle(String title) {
    getWindow().setHeadingText(title);
  }

  @Override
  public void setValue(String value) {
    getWorksheet().setValue(value);
    displayGrid();
    getSpreadsheetToolBar().setDimensions(getWorksheet().getRowCount(), getWorksheet().getColumnCount());
  }

  private TextField createCellEditorTextField() {
    TextField cellEditorTextField = new TextField();
    cellEditorTextField.setSelectOnFocus(true);
    cellEditorTextField.setToolTip(CELL_TOOL_TIP);
    cellEditorTextField.addValueChangeHandler(new ValueChangeHandler<String>() {
      @Override
      public void onValueChange(ValueChangeEvent<String> event) {
        CellSelection<Row> cell = getCellSelectionModel().getSelectCell();
        String cellName = SpreadsheetUtilities.getCellName(cell.getRow() + 1, cell.getCell() - 1);
        getSpreadsheetToolBar().setCellValue(cellName, event.getValue());
      }
    });
    return cellEditorTextField;
  }

  private void displayGrid() {
    if (getVerticalLayoutContainer().getWidgetCount() == 1) {
      getVerticalLayoutContainer().add(getGrid(), new VerticalLayoutData(1, 1));
    } else {
      columnModel = null;
      columnConfigs = null;
      getGrid().reconfigure(getWorksheet().getListStore(), getColumnModel());
    }
  }

  private void displaySelectedCell(List<CellSelection<Row>> cellSelections) {
    String name = null;
    String value = null;
    if (cellSelections != null) {
      if (cellSelections.size() > 0) {
        CellSelection<Row> cellSelection = cellSelections.get(0);
        name = getWorksheet().getNameAbsolute(cellSelection.getRow(), cellSelection.getCell());
        if (name != null) {
          value = getWorksheet().getValueAbsolute(cellSelection.getRow(), cellSelection.getCell());
        }
      }
    }
    getSpreadsheetToolBar().setCellValue(name, value);
  }

  private CellSelectionModel<Row> getCellSelectionModel() {
    if (cellSelectionModel == null) {
      cellSelectionModel = new CellSelectionModel<Row>();
      cellSelectionModel.setSelectionMode(SelectionMode.MULTI);
      cellSelectionModel.addCellSelectionChangedHandler(new CellSelectionChangedHandler<Row>() {
        @Override
        public void onCellSelectionChanged(CellSelectionChangedEvent<Row> event) {
          List<CellSelection<Row>> cellSelections = event.getSelection();
          displaySelectedCell(cellSelections);
        }
      });
    }
    return cellSelectionModel;
  }

  private List<ColumnConfig<Row, ?>> getColumnConfigs() {
    if (columnConfigs == null) {
      columnConfigs = new LinkedList<ColumnConfig<Row, ?>>();
      columnConfigs.add(getRowNumberer());
      int columnCount = getWorksheet().getColumnCount();
      for (int i = 0; i < columnCount; i++) {
        ColumnConfig<Row, String> columnConfig = new ColumnConfig<Row, String>(new RowValueProvider(i));
        columnConfig.setHeader(SpreadsheetUtilities.getColumnName(i));
        columnConfig.setWidth(DEFAULT_COLUMN_WIDTH);
        columnConfig.setCell(getDisplayCell());
        getGridInlineEditing().addEditor(columnConfig, createCellEditorTextField());
        columnConfigs.add(columnConfig);
      }
    }
    return columnConfigs;
  }

  private ColumnModel<Row> getColumnModel() {
    if (columnModel == null) {
      columnModel = new ColumnModel<Row>(getColumnConfigs());
      columnModel.addColumnMoveHandler(new ColumnMoveHandler() {
        @Override
        public void onColumnMove(ColumnMoveEvent event) {
          getWorksheet().renameColumns();
          getSpreadsheetPresenter().onSave();
        }
      });
    }
    return columnModel;
  }

  private Cell<String> getDisplayCell() {
    if (displayCell == null) {
      displayCell = new AbstractCell<String>() {
        @Override
        public void render(Context context, String value, SafeHtmlBuilder sb) {
          if (value != null) {
            if (value.startsWith("=")) {
              sb.appendHtmlConstant("<b>" + getWorksheet().evaluate(value, context.getIndex(), context.getColumn())
                  + "</b>");
            } else {
              sb.appendEscaped(value);
            }
          }
        }
      };
    }
    return displayCell;
  }

  private Grid<Row> getGrid() {
    if (grid == null) {
      grid = new Grid<Row>(getWorksheet().getListStore(), getColumnModel());
      grid.setSelectionModel(getCellSelectionModel());
      grid.getView().setStripeRows(true);
      grid.getView().setColumnLines(true);
      grid.getView().setAutoFill(true);
      grid.setColumnReordering(true);
      getRowNumberer().initPlugin(grid);
      getGridInlineEditing().setEditableGrid(grid);
      getGridDragSource();
      getGridDropTarget();
    }
    return grid;
  }

  private GridDragSource<Row> getGridDragSource() {
    if (gridDragSource == null) {
      gridDragSource = new GridDragSource<Row>(getGrid());
    }
    return gridDragSource;
  }

  private GridDropTarget<Row> getGridDropTarget() {
    if (gridDropTarget == null) {
      gridDropTarget = new GridDropTarget<Row>(getGrid()) {
        @Override
        protected void onDragDrop(DndDropEvent e) {
          getGrid().getStore().clearSortInfo();
          getGrid().getView().getHeader().refresh();
          super.onDragDrop(e);
          getSpreadsheetPresenter().onSave();
        }
      };
      gridDropTarget.setAllowSelfAsSource(true);
      gridDropTarget.setFeedback(Feedback.BOTH);
    }
    return gridDropTarget;
  }

  private GridInlineEditing<Row> getGridInlineEditing() {
    if (gridInlineEditing == null) {
      gridInlineEditing = new GridInlineEditing<Row>(null);
      gridInlineEditing.setClicksToEdit(ClicksToEdit.TWO);
      gridInlineEditing.addCompleteEditHandler(new CompleteEditHandler<Row>() {
        @Override
        public void onCompleteEdit(CompleteEditEvent<Row> event) {
          // Give the change a chance to propagate to model and store
          // Could also use cellEditorTextField's onValueChange method
          Scheduler.get().scheduleFixedDelay(new RepeatingCommand() {
            @Override
            public boolean execute() {
              getSpreadsheetPresenter().onSave();
              getGrid().getView().refresh(false);
              return false;
            }
          }, 250);
        }
      });
    }
    return gridInlineEditing;
  }

  private IdentityValueProvider<Row> getIdentityValueProvider() {
    if (identityValueProvider == null) {
      identityValueProvider = new IdentityValueProvider<Row>();
    }
    return identityValueProvider;
  }

  private RowNumberer<Row> getRowNumberer() {
    if (rowNumberer == null) {
      rowNumberer = new RowNumberer<Row>(getIdentityValueProvider());
    }
    return rowNumberer;
  }

  private SpreadsheetPresenter getSpreadsheetPresenter() {
    return spreadsheetPresenter;
  }

  private SpreadsheetToolBar getSpreadsheetToolBar() {
    if (spreadsheetToolBar == null) {
      spreadsheetToolBar = new SpreadsheetToolBar(getSpreadsheetPresenter());
    }
    return spreadsheetToolBar;
  }

  private VerticalLayoutContainer getVerticalLayoutContainer() {
    if (verticalLayoutContainer == null) {
      verticalLayoutContainer = new VerticalLayoutContainer() {
        @Override
        public void add(Widget child) {
          super.add(child);
          doLayout();
        }
      };
      verticalLayoutContainer.setBorders(true);
      verticalLayoutContainer.add(getSpreadsheetToolBar(), new VerticalLayoutData(1, -1));
    }
    return verticalLayoutContainer;
  }

  private Window getWindow() {
    if (window == null) {
      window = new Window();
      window.getHeader().setIcon(Images.getImageResources().table());
      window.setMinimizable(true);
      window.setMaximizable(true);
      window.setPixelSize(500, 400);
      window.setOnEsc(false);
      window.setWidget(getVerticalLayoutContainer());
      window.addHideHandler(this);
      window.addShowHandler(this);
    }
    return window;
  }

}
